@@ -5,17 +5,53 @@ from typing import (
 )
 
 import torch
+from typing_extensions import Protocol, runtime_checkable
 
 import pytorch_pfn_extras as ppe
 from pytorch_pfn_extras import reporting
 from pytorch_pfn_extras.handler._logic import BaseLogic
+from pytorch_pfn_extras.training import Evaluator, Trainer
 
 if TYPE_CHECKING:
-    from pytorch_pfn_extras.training._trainer import _Trainer
-    from pytorch_pfn_extras.training._evaluator import _Evaluator
     from pytorch_pfn_extras.runtime import BaseRuntime
 
 
+@runtime_checkable
+class DeferredResult(Protocol):
+    """
+    Protocol that is used for models that work asynchronously
+    """
+    def done(self) -> bool:
+        """Returns `True` when a deferred result is available."""
+        raise NotImplementedError("done must be implemented")
+
+    def wait(self) -> 'DeferredResult':
+        """Does a blocking wait until the result is available.
+
+        Returns self to allow chaining calls once the wait is over.
+        """
+        raise NotImplementedError("wait must be implemented")
+
+    def get(self) -> Optional[Any]:
+        """Returns the result when its available. ``None`` if its not ready"""
+        raise NotImplementedError("get must be implemented")
+
+
+class _DeferredIteration:
+
+    def __init__(
+            self,
+            deferred: DeferredResult,
+            idx: int,
+            batch: Dict[str, Any],
+            cback: Callable[..., None],
+    ) -> None:
+        self.deferred = deferred
+        self.idx = idx
+        self.batch = batch
+        self.cback = cback
+
+
 class BaseHandler:
 
     def __init__(
@@ -51,7 +87,7 @@ class BaseHandler:
         """
         pass
 
-    def train_setup(self, trainer: '_Trainer', loader: Iterable[Any]) -> None:
+    def train_setup(self, trainer: Trainer, loader: Iterable[Any]) -> None:
         """A method called only once when starting a training run.
 
         .. seealso:
@@ -63,7 +99,7 @@ class BaseHandler:
 
     def train_epoch_begin(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             loader: Iterable[Any]
     ) -> None:
         """A method called when starting a new epoch.
@@ -75,7 +111,7 @@ class BaseHandler:
         # Called when starting a new epoch.
         pass
 
-    def train_epoch_end(self, trainer: '_Trainer') -> None:
+    def train_epoch_end(self, trainer: Trainer) -> None:
         """A method called when finishing an epoch.
 
         .. seealso:
@@ -87,8 +123,8 @@ class BaseHandler:
 
     def train_validation_begin(
             self,
-            trainer: '_Trainer',
-            evaluator: '_Evaluator',
+            trainer: Trainer,
+            evaluator: Evaluator,
     ) -> None:
         """A method called when starting a validation.
 
@@ -102,8 +138,8 @@ class BaseHandler:
 
     def train_validation_end(
             self,
-            trainer: '_Trainer',
-            evaluator: '_Evaluator',
+            trainer: Trainer,
+            evaluator: Evaluator,
     ) -> None:
         """A method called after validation.
 
@@ -118,7 +154,7 @@ class BaseHandler:
 
     def train_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             batch_idx: int,
             batch: Any,
             complete_fn: Callable[[int, Any], None],
@@ -134,7 +170,7 @@ class BaseHandler:
 
     def train_post_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             batch_idx: int,
             batch: Any,
             outputs: Any,
@@ -150,7 +186,7 @@ class BaseHandler:
 
     def eval_setup(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             loader: Iterable[Any]
     ) -> None:
         """A method called only once when starting a training run.
@@ -164,7 +200,7 @@ class BaseHandler:
         # given.
         pass
 
-    def eval_loop_begin(self, evaluator: '_Evaluator') -> None:
+    def eval_loop_begin(self, evaluator: Evaluator) -> None:
         """A method called before each evaluation step.
 
         Args:
@@ -176,7 +212,7 @@ class BaseHandler:
 
     def eval_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             batch_idx: int,
             batch: Any,
             complete_fn: Callable[[int, Any], None],
@@ -190,7 +226,7 @@ class BaseHandler:
         # Do an evaluation iteration.
         pass
 
-    def eval_loop_end(self, evaluator: '_Evaluator') -> None:
+    def eval_loop_end(self, evaluator: Evaluator) -> None:
         """A method called after running all steps of the evaluation.
 
         .. seealso:
@@ -202,7 +238,7 @@ class BaseHandler:
 
     def eval_post_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             batch_idx: int,
             batch: Any,
             outputs: Any,
@@ -218,7 +254,7 @@ class BaseHandler:
 
 
 ModulesTuple = Tuple[str, torch.nn.Module, 'BaseRuntime']
-PendingIters = List[Tuple[int, Any, Callable[..., None]]]
+PendingIters = List[_DeferredIteration]
 
 
 class Handler(BaseHandler):
@@ -244,8 +280,6 @@ class Handler(BaseHandler):
                     A list of names of outputs that are given as inputs
                     of ``reporting.report`` after each training step.
                     Default is an empty list.
-                * ``'async'`` (bool):
-                    If ``True``, async mode is enabled. Default is ``False``.
         """
         super().__init__(logic, options)
         self.pending_iters: Dict[str, PendingIters] = defaultdict(list)
@@ -258,7 +292,8 @@ class Handler(BaseHandler):
         super().consume_options(options)
         self._eval_report_keys = options.pop('eval_report_keys', [])
         self._train_report_keys = options.pop('train_report_keys', [])
-        self._async = options.pop('async', False)
+        # Consume this argument for backward compatibility
+        options.pop('async', False)
 
     def _runtime_iterator(
             self,
@@ -302,14 +337,7 @@ class Handler(BaseHandler):
             load = loaders.get(sn, None)
             rt.initialize_module(sm, load, optim)
 
-        # Split model can't be used with async unless
-        # we just rewrite the model forward to defer part of its execution
-        # until the result is available
-        if self._async and len(self._ppe_modules) != 1:
-            raise RuntimeError("Async mode is not supported in models "
-                               "splitted across different devices")
-
-    def train_setup(self, trainer: '_Trainer', loader: Iterable[Any]) -> None:
+    def train_setup(self, trainer: Trainer, loader: Iterable[Any]) -> None:
         """A method called only once when starting a training run.
 
         Args:
@@ -322,7 +350,7 @@ class Handler(BaseHandler):
 
     def train_epoch_begin(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             loader: Iterable[Any]
     ) -> None:
         """A method called when starting a new epoch.
@@ -336,27 +364,28 @@ class Handler(BaseHandler):
 
         self._logic.train_epoch_begin(trainer.models, trainer.epoch, loader)
 
-    def train_epoch_end(self, trainer: '_Trainer') -> None:
+    def train_epoch_end(self, trainer: Trainer) -> None:
         """A method called when finishing an epoch.
 
         Args:
             trainer (Trainer): The trainer that calls this method.
         """
-        if self._async:
-            while self.pending_iters:
-                # TODO(ecastill) block until we get the result
-                for sn, sm, rt in self._runtime_iterator(trainer.models):
-                    outs = rt.get_pending_result(sm, True)
-                    if outs is not None:
-                        self._complete_train_step(
-                            trainer, outs, True, sn, sm, rt)
+        while self.pending_iters:
+            # TODO(ecastill) block until we get the result
+            for sn, sm, rt in self._runtime_iterator(trainer.models):
+                p_iter = self.pending_iters[sn][0]
+                if not p_iter.deferred.done():
+                    p_iter.deferred.wait()
+                outs = p_iter.deferred.get()
+                self._complete_train_step(
+                    trainer, outs, True, sn, sm, rt)
 
         self._logic.train_epoch_end(trainer.models, trainer.epoch)
 
     def train_validation_begin(
             self,
-            trainer: '_Trainer',
-            evaluator: '_Evaluator',
+            trainer: Trainer,
+            evaluator: Evaluator,
     ) -> None:
         """A method called when starting a validation.
 
@@ -370,8 +399,8 @@ class Handler(BaseHandler):
 
     def train_validation_end(
             self,
-            trainer: '_Trainer',
-            evaluator: '_Evaluator',
+            trainer: Trainer,
+            evaluator: Evaluator,
     ) -> None:
         """A method called after validation.
 
@@ -389,23 +418,23 @@ class Handler(BaseHandler):
         self._logic.train_validation_end(evaluator.models)
 
     def _complete_train_step(
-            self, trainer: '_Trainer', outs: Any, block: bool,
+            self, trainer: Trainer, outs: Any, block: bool,
             sn: str, sm: torch.nn.Module, rt: 'BaseRuntime',
     ) -> None:
-        idx, batch, cback = self.pending_iters[sn][0]
+        p_iter = self.pending_iters[sn][0]
         self.pending_iters[sn] = self.pending_iters[sn][1:]
         # Since async mode is not supported with device splitting
-        # we now that there is only ONE submodule, so we
+        # we know that there is only ONE submodule, so we
         # can asure that now we can step the optimizers
         self._logic.train_step_optimizers(
-            trainer.models, trainer.optimizers, idx)
+            trainer.models, trainer.optimizers, p_iter.idx)
         if len(self.pending_iters[sn]) == 0:
             del self.pending_iters[sn]
-        cback(idx, outs, is_deferred=block)
+        p_iter.cback(p_iter.idx, outs, is_deferred=block)
 
     def train_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             batch_idx: int,
             batch: Any,
             complete_fn: Callable[[int, Any], None],
@@ -428,18 +457,24 @@ class Handler(BaseHandler):
         outs = self._logic.train_step(
             trainer.models, trainer.optimizers, batch_idx, batch)
 
-        if self._async:
+        if isinstance(outs, DeferredResult):
             # async returns inmediately
             # Check if there is a completed iteration
             # If we enqueue everything first, we will blow up with
             # memory due to the dataloaders being in the background
             # We need to call the tagged runtimes since async mode
             # require different treatment depending on the device.
+            if len(self._ppe_modules) != 1:
+                raise RuntimeError("Async mode is not supported in models "
+                                   "splitted across different devices")
             for sn, sm, rt in self._runtime_iterator(trainer.models):
-                self.pending_iters[sn].append((batch_idx, batch, complete_fn))
-                outs = rt.get_pending_result(sm, False)
-                if outs is not None:
-                    self._complete_train_step(trainer, outs, False, sn, sm, rt)
+                self.pending_iters[sn].append(
+                    _DeferredIteration(outs, batch_idx, batch, complete_fn))
+                p_iter = self.pending_iters[sn][0]
+                if p_iter.deferred.done():
+                    t_outs = p_iter.deferred.get()
+                    self._complete_train_step(
+                        trainer, t_outs, False, sn, sm, rt)
         else:
             self._logic.train_step_optimizers(
                 trainer.models, trainer.optimizers, batch_idx)
@@ -447,7 +482,7 @@ class Handler(BaseHandler):
 
     def eval_setup(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             loader: Iterable[Any]
     ) -> None:
         """Called only once when starting a training run.
@@ -462,19 +497,19 @@ class Handler(BaseHandler):
         self._setup(evaluator.models, loader)
 
     def _complete_eval_step(
-            self, evaluator: '_Evaluator', outs: Any, block: bool,
+            self, evaluator: Evaluator, outs: Any, block: bool,
             sn: str, sm: torch.nn.Module, rt: 'BaseRuntime',
     ) -> None:
         # This call is deferred
-        idx, batch, cback = self.pending_iters[sn][0]
+        p_iter = self.pending_iters[sn][0]
         self.pending_iters[sn] = self.pending_iters[sn][1:]
         if len(self.pending_iters[sn]) == 0:
             del self.pending_iters[sn]
-        cback(idx, outs, is_deferred=block)
+        p_iter.cback(p_iter.idx, outs, is_deferred=block)
 
     def eval_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             batch_idx: int,
             batch: Any,
             complete_fn: Callable[[int, Any], None],
@@ -495,23 +530,28 @@ class Handler(BaseHandler):
 
         outs = self._logic.eval_step(evaluator.models, batch_idx, batch)
 
-        if self._async:
+        if isinstance(outs, DeferredResult):
             # Is async returns inmediately
             # Check if there is a completed iteration
             # If we enqueue everything first, we will blow up with
             # memory due to the dataloaders being in the background
+            if len(self._ppe_modules) != 1:
+                raise RuntimeError("Async mode is not supported in models "
+                                   "splitted across different devices")
             for sn, sm, rt in self._runtime_iterator(evaluator.models):
-                self.pending_iters[sn].append((batch_idx, batch, complete_fn))
-                outs = rt.get_pending_result(sm, False)
-                if outs is not None:
+                self.pending_iters[sn].append(
+                    _DeferredIteration(outs, batch_idx, batch, complete_fn))
+                p_iter = self.pending_iters[sn][0]
+                if p_iter.deferred.done():
+                    t_outs = p_iter.deferred.get()
                     self._complete_eval_step(
-                        evaluator, outs, False, sn, sm, rt)
+                        evaluator, t_outs, False, sn, sm, rt)
         else:
             complete_fn(batch_idx, outs)
 
     def eval_post_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             batch_idx: int,
             batch: Any,
             outputs: Any,
@@ -532,23 +572,23 @@ class Handler(BaseHandler):
         for out in self._eval_report_keys:
             reporting.report({"val/{}".format(out): outputs[out]})
 
-    def eval_loop_end(self, evaluator: '_Evaluator') -> None:
+    def eval_loop_end(self, evaluator: Evaluator) -> None:
         """A method called after running all steps of the evaluation.
 
         Args:
             evaluator (Evaluator): The evaluator.
         """
-        if self._async:
-            while self.pending_iters:
-                for sn, sm, rt in self._runtime_iterator(evaluator.models):
-                    outs = rt.get_pending_result(sm, True)
-                    if outs is not None:
-                        self._complete_eval_step(
-                            evaluator, outs, True, sn, sm, rt)
+        while self.pending_iters:
+            for sn, sm, rt in self._runtime_iterator(evaluator.models):
+                p_iter = self.pending_iters[sn][0]
+                if not p_iter.deferred.done():
+                    p_iter.deferred.wait()
+                outs = p_iter.deferred.get()
+                self._complete_eval_step(evaluator, outs, True, sn, sm, rt)
 
     def train_post_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             batch_idx: int,
             batch: Any,
             outputs: Any
