@@ -1,4 +1,4 @@
-# mypy: ignore-errors
+from typing import Any, Optional
 
 import torch
 
@@ -6,7 +6,7 @@ from pytorch_pfn_extras.nn.modules.lazy import UninitializedParameter
 from pytorch_pfn_extras.nn.modules.lazy import LazyInitializationMixin
 
 
-class LazyLinear(LazyInitializationMixin, torch.nn.Linear):
+class LazyLinear(LazyInitializationMixin, torch.nn.Linear):  # type: ignore[misc]
     """Linear module with lazy weight initialization.
 
     When ``in_features`` is ``None``, it is determined at the first time of
@@ -15,13 +15,13 @@ class LazyLinear(LazyInitializationMixin, torch.nn.Linear):
 
     lazy_parameter_names = ('weight',)
 
-    def __init__(self, in_features, *args, **kwargs):
+    def __init__(self, in_features: Optional[int], *args: Any, **kwargs: Any) -> None:
         super().__init__(in_features or 0, *args, **kwargs)
         if in_features is None:
-            self.in_features = None
+            self.in_features = None  # type: ignore[assignment]
             self.weight = UninitializedParameter()
 
-    def forward(self, input):
+    def forward(self, input: torch.Tensor) -> torch.Tensor:
         if isinstance(self.weight, UninitializedParameter):
             self.in_features = input.shape[-1]
             self.weight = torch.nn.Parameter(self.weight.new_empty(
@@ -29,7 +29,7 @@ class LazyLinear(LazyInitializationMixin, torch.nn.Linear):
             self.reset_parameters()
         return super().forward(input)
 
-    def reset_parameters(self):
+    def reset_parameters(self) -> None:
         # Defer initialization of parameters until shape of the parameter
         # is determiend.
         if self.lazy_parmeters_determined:
