@@ -18,6 +18,7 @@ from torch.onnx.utils import \
     _export as torch_export, _model_to_graph as torch_model_to_graph
 
 from pytorch_pfn_extras.onnx import _as_output as as_output
+from pytorch_pfn_extras.onnx import _grad as grad
 from pytorch_pfn_extras.onnx.annotate import init_annotate
 from pytorch_pfn_extras.onnx.strip_large_tensor import \
     LARGE_TENSOR_DATA_THRESHOLD
@@ -118,7 +119,8 @@ def _export(
         opset_ver = _default_onnx_opset_version
     strip_doc_string = kwargs.pop('strip_doc_string', True)
     with init_annotate(model, opset_ver) as ann, \
-            as_output.trace(model) as (model, outputs):
+            as_output.trace(model) as (model, outputs), \
+            grad.init_grad_state():
         outs = _export_util(
             model, args, bytesio, strip_doc_string=False, **kwargs)
         onnx_graph = onnx.load(io.BytesIO(bytesio.getvalue()))
