@@ -1,19 +1,17 @@
 # mypy: ignore-errors
 
 from typing import (
-    Any, Dict, Generator, Iterable, List, Optional, Tuple, Union, TYPE_CHECKING
+    Any, Dict, Generator, Iterable, Optional, Tuple, Union
 )
 
 import torch
 
+from pytorch_pfn_extras.training import Evaluator, Trainer
+
 _RUNTIME_TAG_NAME = '_ppe_runtime'
 
 DeviceLike = Union[str, torch.device]
 
-if TYPE_CHECKING:
-    from pytorch_pfn_extras.training._trainer import _Trainer
-    from pytorch_pfn_extras.training._evaluator import _Evaluator
-
 
 class BaseRuntime:
     """A base class for collections of device-specific callback functions.
@@ -127,7 +125,7 @@ class BaseRuntime:
 
     def train_pre_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -138,7 +136,7 @@ class BaseRuntime:
         (typically one) iterations and an update.
 
         Args:
-            trainer (_Trainer): A trainer.
+            trainer (Trainer): A trainer.
             module (torch.nn.Module): A module.
             batch_idx (int): The batch index.
             batch (list of torch.Tensor):
@@ -150,7 +148,7 @@ class BaseRuntime:
 
     def train_post_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -162,7 +160,7 @@ class BaseRuntime:
         (typically one) iterations and an update.
 
         Args:
-            trainer (_Trainer): A trainer.
+            trainer (Trainer): A trainer.
             module (torch.nn.Module): A module.
             batch_idx (int): The batch index.
             batch (list of torch.Tensor):
@@ -196,7 +194,7 @@ class BaseRuntime:
 
     def eval_pre_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -204,7 +202,7 @@ class BaseRuntime:
         """The method called at the beginning of each evaluation.
 
         Args:
-            evaluator (_Evaluator): An evaluator.
+            evaluator (Evaluator): An evaluator.
             module (torch.nn.Module): A module.
             batch_idx (int): The batch index.
             batch (list of torch.Tensor):
@@ -216,7 +214,7 @@ class BaseRuntime:
 
     def eval_post_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -225,7 +223,7 @@ class BaseRuntime:
         """The method called at the end of each evaluation.
 
         Args:
-            evaluator (_Evaluator): An evaluator.
+            evaluator (Evaluator): An evaluator.
             module (torch.nn.Module): A module.
             batch_idx (int): The batch index.
             batch (list of torch.Tensor):
@@ -237,25 +235,6 @@ class BaseRuntime:
         """
         raise NotImplementedError()
 
-    def get_pending_result(
-            self,
-            module: torch.nn.Module,
-            blocking: bool,
-    ) -> List[torch.Tensor]:
-        """The method called to retrieve the result of a asynchronous call.
-
-        Args:
-            module (torch.nn.Module): A module.
-            blocking (bool): A flag to determine wether to wait for
-                an asynchronous call completion or returns None.
-
-        Returns: (list of torch.Tensor):
-            The list of output tensors of the oldest async operation.
-            Will return `None` if `blocking=False` and no asynchronous
-            call is complete.
-        """
-        raise NotImplementedError()
-
 
 class PyTorchRuntime(BaseRuntime):
     """A collections of callback functions for the devices that PyTorch
@@ -290,7 +269,7 @@ class PyTorchRuntime(BaseRuntime):
 
     def train_pre_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -299,7 +278,7 @@ class PyTorchRuntime(BaseRuntime):
 
     def train_post_step(
             self,
-            trainer: '_Trainer',
+            trainer: Trainer,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -309,7 +288,7 @@ class PyTorchRuntime(BaseRuntime):
 
     def eval_pre_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -318,7 +297,7 @@ class PyTorchRuntime(BaseRuntime):
 
     def eval_post_step(
             self,
-            evaluator: '_Evaluator',
+            evaluator: Evaluator,
             module: torch.nn.Module,
             batch_idx: int,
             batch: Any,
@@ -326,13 +305,6 @@ class PyTorchRuntime(BaseRuntime):
     ) -> None:
         pass
 
-    def get_pending_result(
-            self,
-            module: torch.nn.Module,
-            blocking: bool,
-    ) -> List[torch.Tensor]:
-        pass
-
 
 def _module_runtime_tag(module: torch.nn.Module) -> BaseRuntime:
     return getattr(  # type: ignore[no-any-return]
