@@ -9,7 +9,7 @@ from pytorch_pfn_extras.training import extension
 from pytorch_pfn_extras.training.extensions import log_report \
     as log_report_module
 from pytorch_pfn_extras.training.extensions import util
-from pytorch_pfn_extras.training.manager import _BaseExtensionsManager
+from pytorch_pfn_extras.training._manager_protocol import ExtensionsManagerProtocol
 
 
 def create_header_and_templates(
@@ -90,10 +90,9 @@ class PrintReport(extension.Extension):
             self._infer_entries = False
         self._entries = entries
         self._log_report = log_report
+        self._log_looker = None
         self._out = out
 
-        self._log_len = 0  # number of observations already printed
-
         # format information
         header, templates = create_header_and_templates(entries)
         self._header: Optional[str] = header  # printed at the first call
@@ -102,7 +101,7 @@ class PrintReport(extension.Extension):
 
     def get_log_report(
             self,
-            manager: _BaseExtensionsManager,
+            manager: ExtensionsManagerProtocol,
     ) -> log_report_module.LogReport:
         log_report = self._log_report
         if isinstance(log_report, str):
@@ -117,10 +116,13 @@ class PrintReport(extension.Extension):
             raise TypeError('log report has a wrong type %s' %
                             type(log_report))
 
+    def initialize(self, manager: ExtensionsManagerProtocol) -> None:
+        log_report = self.get_log_report(manager)
+        self._log_looker = log_report._log_buffer.emit_new_looker()
+
     def _update_entries(self, log_report: log_report_module.LogReport) -> None:
-        log = log_report.log
         updated_flag = False
-        aggregate_entries = log[self._log_len:]
+        aggregate_entries = self._log_looker.get()
         for obs in aggregate_entries:
             for entry in obs.keys():
                 if entry not in self._all_entries:
@@ -140,9 +142,8 @@ class PrintReport(extension.Extension):
             self._header = header  # printed at the first call
             self._templates = templates
 
-    def __call__(self, manager: _BaseExtensionsManager) -> None:
+    def __call__(self, manager: ExtensionsManagerProtocol) -> None:
         log_report = self.get_log_report(manager)
-        log = log_report.log
 
         if self._infer_entries:
             # --- update entries ---
@@ -154,16 +155,14 @@ class PrintReport(extension.Extension):
             out.write(self._header)
             self._header = None
 
-        log_len = self._log_len
-        while len(log) > log_len:
+        for line in self._log_looker.get():
             # delete the printed contents from the current cursor
             if os.name == 'nt':
                 util.erase_console(0, 0)
             else:
                 out.write('\033[J')
-            self._print(log[log_len])
-            log_len += 1
-        self._log_len = log_len
+            self._print(line)
+        self._log_looker.clear()
 
     def state_dict(self) -> Dict[str, Any]:
         log_report = self._log_report
